/* Citadel/UX Color module - By Jiva DeVoe (C) Copyright 1995 Jiva DeVoe */

/* Foreground colors */

#include <stdlib.h>

#define GREEN 32
#define RED 31
#define YELLOW 33
#define BLUE 34
#define BLACK 30
#define MAGENTA 35
#define CYAN 36
#define WHITE 37

/* Background Colors */

#define B_GREEN 42
#define B_RED 41
#define B_YELLOW 43
#define B_BLUE 44
#define B_BLACK 40
#define B_MAGENTA 45
#define B_CYAN 46
#define B_WHITE 47


setcolor(fore,back)
int fore;
int back;
{
	if(!(strucmp(getenv("TERM"),"ansi")))
		printf("[%d;%dm",fore,back);
}
