/* Call log routines */

#include <stdio.h>
#include "citadel.h"

logit(string)
char *string;
{
long curtime;
FILE *mylog;
char now[80];

	time(&curtime);
/*	strcpy(now, asctime(curtime)); */
	mylog=fopen("/tmp/cit.log","a");
	fprintf(mylog,"%ld %s: %s\n",curtime,usersupp.fullname,string);
	fclose(mylog);
	
}

logint(string)
int string;
{
long curtime;
FILE *mylog;
char now[80];

	time(&curtime);
/*	strcpy(now, asctime(curtime)); */
	mylog=fopen("/tmp/cit.log","a");
	fprintf(mylog,"%ld %s: %c\n",curtime,usersupp.fullname,string);
	fclose(mylog);
	
}
