/****************************************************************************/
/*                  YOUR SYSTEM CONFIGURATION                               */
/* Set all the values in this file appropriately BEFORE compiling any of the*/
/* C programs. If you are upgrading from an older version of Citadel/UX, it */
/* is vitally important that the #defines which are labelled "structure size*/
/* variables" are EXACTLY the same as they were in your old system,         */
/* otherwise your files will be munged beyond repair.                       */
/****************************************************************************/

/* NOTE THAT THIS FILE IS MUCH, MUCH SMALLER THAN IT USED TO BE.
 * That's because the setup program now creates a citadel.config file with
 * all of the settings that don't really need to be in a header file.
 * You can now run setup whenever you want, and change lots of parameters
 * without having to recompile the whole system!
 */

#define BBSDIR		"/usr/citadel"
/* This is the directory Citadel will reside in. */

/*** STRUCTURE SIZE VARIABLES ***/
/* You may NOT change these six values once you set up your system.	    */
#define MAXROOMS	64		/* Number of rooms in system        */
#define MAILSLOTS	35		/* Number of mail slots             */
#define MSGSPERRM	150		/* Messages per room                */
#define CALLLOG		500		/* Number of entries in call log    */
/* Do not set MAILSLOTS higher than MSGSPERRM 				    */
/*** END OF STRUCTURE SIZE VARIABLES ***/
